<?php

/**
 * A phpBB DBAL store.
 *
 * @package phpbb-openid
 */

/**
 * Require the base class file.
 */
require_once "{$openid_root_path}SQLStore.$phpEx";

/**
 * Require the phpBB DBAL connection file.
 */
require_once "{$openid_root_path}BBDatabaseConnection.$phpEx";

class Auth_OpenID_BBStore extends Auth_OpenID_SQLStore
{
	var $sessions_data_table_name;

	function Auth_OpenID_BBStore()
	{
		global $table_prefix;
		$openid_table_prefix = $table_prefix . 'oid_';

		$connection = new Auth_OpenID_BBDatabaseConnection();
		$settings_table_name = $openid_table_prefix . 'settings';
		$associations_table_name = $openid_table_prefix . 'associations';
		$nonces_table_name = $openid_table_prefix . 'nonces';
		$sessions_data_table_name = $openid_table_prefix . 'sessions_data';

		$this->sessions_data_table_name = $sessions_data_table_name;
		parent::Auth_OpenID_SQLStore($connection, $settings_table_name, $associations_table_name, $nonces_table_name);
	}	

	function isError($value)
	{
		return !$value;
	}

	function setSQL()
	{
		// TODO: These are supposed to be SPs. Once they are, remove quotes.
		$this->sql = array(
			'nonce_table' => '%s',
			
			'assoc_table' => '%s',
			
			'settings_table' => '%s',
			
			'sessions_data_table' => '%s',

			'create_auth' =>
				'INSERT INTO %s VALUES (\'auth_key\', \'?\')',
			
			'get_auth' =>
				'SELECT value FROM %s WHERE setting = \'auth_key\'',
			
			'set_assoc' =>
				'REPLACE INTO %s VALUES (\'?\', \'?\', \'?\', ?, ?, \'?\')',
			
			'get_assocs' =>
				'SELECT handle, secret, issued, lifetime, assoc_type FROM %s '.
				'WHERE server_url = \'?\'',
			
			'get_assoc' =>
				'SELECT handle, secret, issued, lifetime, assoc_type FROM %s '.
				'WHERE server_url = \'?\' AND handle = \'?\'',
			
			'remove_assoc' =>
				'DELETE FROM %s WHERE server_url = \'?\' AND handle = \'?\'',
			
			'add_nonce' =>
				'REPLACE INTO %s (nonce, expires) VALUES (\'?\', ?)',
			
			'get_nonce' =>
				'SELECT * FROM %s WHERE nonce = \'?\'',
			
			'remove_nonce' =>
				'DELETE FROM %s WHERE nonce = \'?\'',
			
			'set_var' =>
				'REPLACE INTO %s (session_id, setting, value)'.
				'VALUES (\'?\', \'?\', \'?\')',
			
			'get_var' =>
				'SELECT value FROM %s WHERE session_id = \'?\' AND setting = \'?\'',

			'del_var' =>
				'DELETE FROM %s WHERE session_id = \'?\' AND setting = \'?\'',
			
			'del_session' =>
				'DELETE FROM %s WHERE session_id = \'?\'',
		);
	}

    function _fixSQL()
    {
		parent::_fixSQL();
		
		$keys = array(
			'set_var',
			'get_var',
			'del_var',
			'del_session',
		);
		
        foreach ($keys as $k)
        {
            $this->sql[$k] = sprintf($this->sql[$k],
				$this->sessions_data_table_name);
        }
    }
    
    function tableExists($table_name)
    {
		return true;
    }

    function createTables()
    {
        return true;
    }
}

?>