<?php

/**
 * A phpBB session.
 *
 * @package phpbb-openid
 */

/**
 * Require the base class file.
 */
require_once "{$yadis_root_path}Manager.$phpEx";

class Services_Yadis_BBSession extends Services_Yadis_PHPSession
{
	var $sid;
	var $store;
	
	function Services_Yadis_BBSession($store, $sid = null)
	{
		if (!$sid)
		{
			global $user;
			$sid = $user->session_id;
		}
		$this->sid = $sid;
		$this->store = $store;
	}
	
    /**
     * Set a session key/value pair.
     *
     * @param string $name The name of the session key to add.
     * @param string $value The value to add to the session.
     */
    function set($name, $value)
    {
		$this->store->connection->query(
			$this->store->sql['set_var'],
			array(
				$this->sid,
				$name,
				serialize($value))
		);
	}

    /**
     * Get a key's value from the session.
     *
     * @param string $name The name of the key to retrieve.
     * @param string $default The optional value to return if the key
     * is not found in the session.
     * @return string $result The key's value in the session or
     * $default if it isn't found.
     */
	function get($name, $default = null)
	{
		$row = $this->store->connection->getRow(
			$this->store->sql['get_var'],
			array(
				$this->sid,
				$name)
		);
		return (!($row)) ? $default : unserialize($row['value']);
	}

    /**
     * Remove a key/value pair from the session.
     *
     * @param string $name The name of the key to remove.
     */
    function del($name)
    {
		$this->store->connection->query(
			$this->store->sql['del_var'],
			array(
				$this->sid,
				$name)
		);
    }

    /**
     * Return the contents of the session in array form.
     */
    function contents()
    {
        return null;
    }
    
    /**
     * Terminates the session.
     */
    function terminate()
    {
		$this->store->connection->query(
			$this->store->sql['del_session'],
			array(
				$this->sid)
		);
    }
}

?>