<?php

/***************************************************************************
 *  functions_openid.php
 *
 */

function openid_finish_auth() {
	global $consumer, $session, $template, $lang, $db, $HTTP_GET_VARS, $HTTP_POST_VARS, $phpEx;

	$response = $consumer->complete($HTTP_GET_VARS);

	if (!$HTTP_POST_VARS['redirect'] && $session->get("long_redirect") && $HTTP_GET_VARS[openid_mode]) {
		$HTTP_POST_VARS['redirect'] = str_replace(".$phpEx&",".$phpEx?", urldecode($session->get("long_redirect")) );
		$session->set("long_redirect", '');
	}

	$message = '';
	switch($response->status) {
		case Auth_OpenID_CANCEL:
			openid_failure($lang['OpenID_canceled']);
			break;
		case Auth_OpenID_FAILURE:
			openid_failure($response->message);
			break;
		case Auth_OpenID_SUCCESS:
			$openid = $response->identity_url;
			$sreg = $response->extensionResponse('sreg');

			$sql = "SELECT * 
					FROM " . USERS_TABLE . "
					WHERE user_openid = '" . str_replace("\'", "''", $openid) . "' ";
			if ( !($result = $db->sql_query($sql)) ) {
				message_die(GENERAL_ERROR, 'Error in obtaining userdata', '', __LINE__, __FILE__, $sql);
			}
			if( $row = $db->sql_fetchrow($result) ) {
				fake_login($row['username']);
			} else {
				reg_openid_user($openid, $sreg);
			}
			break;
	}
}

function openid_begin_auth($openid_url, $redirect) {
	global $board_config, $lang, $phpEx, $consumer, $session, $userdata;	

	//Manage Long--Redirect
	if (!empty($redirect)) {
		$session->set("long_redirect", $redirect);
	}

	//Create trust-root
	$server_protocol = ($board_config['cookie_secure']) ? 'https://' : 'http://';
	$server_name = preg_replace('#^\/?(.*?)\/?$#', '\1', trim($board_config['server_name']));
	$server_port = ($board_config['server_port'] <> 80) ? ':' . trim($board_config['server_port']) : '';
	$script_name = preg_replace('#^\/?(.*?)\/?$#', '\1', trim($board_config['script_path']));
	$script_name = ($script_name == '') ? $script_name : '/' . $script_name;

	$trust_root = $server_protocol . $server_name . $server_port . $script_name . '/';
	$process_url = $trust_root . 'login.' . $phpEx . '?redirect=' . $redirect;
	
	// Begin the OpenID authentication process.
	if ($openid_url[0] != '=' && $openid_url[1] != '@' && strpos($openid_url, '.') === false) {
		$openid_url = sprintf('http://%s.phpbb.cc/', $openid_url);
	}
	$auth_request = $consumer->begin($openid_url);

	// Handle failure status return values.
	if (!$auth_request) {
		openid_failure($lang['OpenID_notfound']);
	}

	$sreg_req = array('nickname', 'email');
	$sreg_opt = array('country', 'timezone', 'language');
	if (isset($userdata['user_fullname'])) {
		$sreg_opt[] = 'fullname';
	}
	if (isset($userdata['user_gender'])) {
		$sreg_opt[] = 'gender';
	}
	if (isset($userdata['user_birthday'])) {
		$sreg_opt[] = 'dob';
	}
	$auth_request->addExtensionArg('sreg', 'required', join(',' ,$sreg_req));
	$auth_request->addExtensionArg('sreg', 'optional', join(',' ,$sreg_opt));
	
	// Redirect the user to the OpenID server for authentication.  Store
	// the token for this authentication so we can verify the response.
	
	$redirect_url = $auth_request->redirectURL($trust_root, $process_url);
	header("Location: ".$redirect_url);
}


// ----------------------------------------------------
// reg_openid_user()
//
// Adds new user to phpBB database, basing on the information
// returned from the OpenID Server.
// ----------------------------------------------------
function reg_openid_user($openid, $sreg) {
	global $db, $board_config, $lang;
	global $phpbb_root_path, $phpEx;

	// Get extra data from OpenID SimpleReg Extension:
	$email = (@$sreg['email']) ? $sreg['email'] : '';
	$nickname = (@$sreg['nickname']) ? $sreg['nickname'] : '';
	
	if ($email != '') {
		$sql = "SELECT user_id, username FROM " . USERS_TABLE . "
			WHERE user_email = '" . str_replace("\'", "''", $email) . "'";
		if ( !($result = $db->sql_query($sql)) ) {
			message_die(GENERAL_ERROR, 'Could not obtain user information', '', __LINE__, __FILE__, $sql);
		}
		if ( ($row = $db->sql_fetchrow($result)) ) {
			if ($nickname == $row['username']) {
				openid_migrate($row['user_id'], $row['username'], $email, $openid);
			} else {
				message_die(GENERAL_MESSAGE, $lang['Email_taken']);
			}
		}
	}

	$aim = '';
	$matches = array();
	if (preg_match('/http:\/\/openid\.aol\.com\/(.+)/i', $openid, $matches)) {
		$aim = $matches[1];
		$sql = "SELECT user_id, username, user_email FROM " . USERS_TABLE . "
			WHERE user_aim = '" . str_replace("\'", "''", $aim) . "' AND user_openid = ''";
		if ( !($result = $db->sql_query($sql)) ) {
			message_die(GENERAL_ERROR, 'Could not obtain user information', '', __LINE__, __FILE__, $sql);
		}
		if ( ($row = $db->sql_fetchrow($result)) && $row['user_email'] != '' ) {
			openid_migrate($row['user_id'], $row['username'], $row['user_email'], $openid);
		}
	}

	$fullname = (@$sreg['fullname']) ? $sreg['fullname'] : '';

	$timezone = $board_config['board_timezone'];
	if (@$sreg['timezone']) {
		include_once("{$phpbb_root_path}includes/openid/timezones.$phpEx");
		$timezone = $timezones[$sreg['timezone']] / 100;
	}

	$location = (@$sreg['country']) ? $sreg['country'] : '';
	if (@$sreg['country']) {
		include_once("{$phpbb_root_path}includes/openid/countries.$phpEx");
		$location = $country_codes[$sreg['country']];
	}

	$language = $board_config['default_lang']; 
	if (@$sreg['language']) {
		include_once("{$phpbb_root_path}includes/openid/languages.$phpEx");
		$language = strtolower($language_codes[$sreg['language']]);
	}

	$gender = 0;
	if (@$sreg['gender']) {
		switch ($sreg['gender']) {
			case 'M':
			case 'm': // PIP
				$gender = 1; // Niels
				break;
			case 'F':
			case 'f': // PIP
				$gender = 2; // Niels
				break;
		}
	}
	
	if (@$sreg['dob']) {
		$bday_year  = intval(substr($sreg['dob'], 0, 4));
		$bday_month = intval(substr($sreg['dob'], 5, 2));
		$bday_day   = intval(substr($sreg['dob'], 8, 2));
	}
	if (function_exists('mkrealdate')) { // Niels
		$birthday = 999999;
		if (@$sreg['dob'] && $bday_year) {
			$birthday = mkrealdate($bday_day, $bday_month, $bday_year);
		}
	}
	else
	{
		$birthday = '';
		if (@$sreg['dob']) {
			$birthday = sprintf('%02d%02d%04d', $bday_month, $bday_day, $bday_year);
		}
	}

	//all this crap to get user_id to reg new user WTF ?!
	$sql = "SELECT MAX(user_id) AS total
		FROM " . USERS_TABLE;
	if ( !($result = $db->sql_query($sql)) || !($row = $db->sql_fetchrow($result)) ) {
		message_die(GENERAL_ERROR, 'Could not obtain next user_id information', '', __LINE__, __FILE__, $sql);
	}
	$user_id = $row['total'] + 1;

	//first try nickname
	if ($nickname == '' ) {
		$nickname = openid_username($openid);
	}
	if ($nickname == '' ) {
		$nickname = $user_id;
	}

	$sql = "SELECT username
		FROM " . USERS_TABLE . "
			WHERE username = '" . str_replace("\'", "''", $nickname) . "'";
	if ( !($result = $db->sql_query($sql)) ) {
		message_die(GENERAL_ERROR, 'Could not obtain username information', '', __LINE__, __FILE__, $sql);
	}
	$username = ($row = $db->sql_fetchrow($result)) ? '' : $nickname;

	for ($i = 2; ; $i++) {
		$sql = "SELECT username
			FROM " . USERS_TABLE . "
				WHERE username = '" . str_replace("\'", "''", $nickname) . "$i'";
		if ( !($result = $db->sql_query($sql)) ) {
			message_die(GENERAL_ERROR, 'Could not obtain username information', '', __LINE__, __FILE__, $sql);
		}
		if (!($row = $db->sql_fetchrow($result))) {
			break;
		}
	}
	if ($username == '') {
		$username = "$nickname$i";
	}

	if ($aim != '') {
		$website = sprintf('http://www.aimpages.com/%s/', $aim);
	} elseif (preg_match('#^http[s]?\\:\\/\\/[a-z0-9\-\_]+\.([a-z0-9\-\_]+\.)?[a-z]+#i', $openid)) {
		$website = $openid;
	} elseif (preg_match('#^[@=].+#i', $openid)) {
		$website = sprintf('http://xri.net/%s/', $openid);
	} else {
		$website = '';
	}
	
	$avatar = '';
	$avatar_type = USER_AVATAR_NONE;
	if ($board_config['allow_avatar_remote'] == REMOTE_AVATAR_AUTO || $board_config['allow_avatar_remote'] == REMOTE_AVATAR_BOTH) {
		include_once("{$phpbb_root_path}includes/functions_pavatar.{$phpEx}");
		include_once("{$phpbb_root_path}includes/usercp_avatar.{$phpEx}");
		$error = null;
		$error_msg = '';
		if (($pavatar_remoteurl = pavatar_detect($openid, $error, $error_msg, false)) != '') {
			user_avatar_url('register', $error, $error_msg, $pavatar_remoteurl);
			if (!$error) {
				$avatar = $pavatar_remoteurl;
				$avatar_type = USER_AVATAR_REMOTE;
			}
		}
	}

	$username = phpbb_clean_username($username);
	
	$fullname_col_sql = ($fullname) ? ', user_fullname' : '';
	$fullname_value_sql = ($fullname) ? ", '" . str_replace("\'", "''", $fullname) . "'" : '';
	$gender_col_sql = ($gender) ? ', user_gender' : '';
	$gender_value_sql = ($gender) ? ", $gender" : '';
	$birthday_col_sql = ($birthday) ? ', user_birthday' : '';
	$birthday_value_sql = ($birthday) ? ", '$birthday'" : '';

	// creating new user
	$sql = 'INSERT INTO ' . USERS_TABLE .
		'( user_id, '
		. 'username, '
		. 'user_openid,'
		. 'user_regdate, '
		. 'user_password, '
		. 'user_email, '
		. 'user_website, '
		. 'user_aim, '
		. 'user_avatar, '
		. 'user_avatar_type, '
		. 'user_occ, '
		. 'user_from, '
		. 'user_sig, '
		. 'user_viewemail, '
		. 'user_attachsig, '
		. 'user_allowsmile, '
		. 'user_allowhtml, '
		. 'user_allowbbcode, '
		. 'user_allow_viewonline, '
		. 'user_notify, '
		. 'user_notify_pm, '
		. 'user_popup_pm, '
		. 'user_timezone, '
		. 'user_dateformat, '
		. 'user_lang, '
		. 'user_style, '
		. 'user_level, '
		. 'user_allow_pm, '
		. 'user_active'
		. $fullname_col_sql
		. $gender_col_sql
		. $birthday_col_sql
		. ')'
	.' VALUES ('
		. "$user_id, "											// user_id
		. "'" . str_replace("\'", "''", $username) . "', "		// username
		. "'" . str_replace("\'", "''", $openid) . "', "		// openid
		. time() . ', '											// user_regdate
		. "'', "												// user_password
		. "'" . str_replace("\'", "''", $email) . "', "			// user_email
		. "'" . str_replace("\'", "''", $website) . "', "		// user_website
		. "'" . str_replace("\'", "''", $aim) . "', "			// user_aim
		. "'" . str_replace("\'", "''", $avatar) . "', "		// user_avatar
		. $avatar_type . ', '									// user_avatar_type
		. "'', "												// user_occ
		. "'" . str_replace("\'", "''", $location) . "', "		// user_from
		. "'', "												// user_sig
		. $board_config['board_email_form'] . ', '				// user_viewemail
		. $board_config['allow_sig'] . ', '						// user_attachsig
		. $board_config['allow_smilies'] . ', '					// user_allowsmile
		. $board_config['allow_html'] . ', '					// user_allowhtml
		. $board_config['allow_bbcode'] . ', '					// user_allowbbcode
		. "1, "													// user_allow_viewonline
		. "1, "													// user_notify
		. "1, "													// user_notify_pm
		. "1, "													// user_popup_pm
		. $timezone . ', '										// user_timezone
		. "'" . $board_config['default_dateformat'] . "', "		// user_dateformat
		. "'" . str_replace("\'", "''", $language) . "', "		// user_lang
		. $board_config['default_style'] . ', '					// user_style
		. "0, "													// user_level
		. "1, "													// user_allow_pm
		. "1"													// user_active
		. $fullname_value_sql
		. $gender_value_sql
		. $birthday_value_sql
		. ")";

	if ( !($result = $db->sql_query($sql)) )
	{
		message_die(GENERAL_ERROR, 'Could not insert data into users table', '', __LINE__, __FILE__, $sql);
	}

	$sql = "INSERT INTO " . GROUPS_TABLE . " (group_name, group_description, group_single_user, group_moderator)
		VALUES ('', 'Personal User', 1, 0)";
	if ( !($result = $db->sql_query($sql)) )
	{
		message_die(GENERAL_ERROR, 'Could not insert data into groups table', '', __LINE__, __FILE__, $sql);
	}

	$group_id = $db->sql_nextid();

	$sql = "INSERT INTO " . USER_GROUP_TABLE . " (user_id, group_id, user_pending)
		VALUES ($user_id, $group_id, 0)";
	if( !($result = $db->sql_query($sql, END_TRANSACTION)) )
	{
		message_die(GENERAL_ERROR, 'Could not insert data into user_group table', '', __LINE__, __FILE__, $sql);
	}
	fake_login($username);
}


function fake_login($username) {
	global $HTTP_POST_VARS;
	
	$username = html_entity_decode(str_replace("'", "\'", trim($username)));
	$username = str_replace("\'", "'", $username);
	
	$HTTP_POST_VARS['login'] = 1;
	$HTTP_POST_VARS['username'] = $username;
	global $openid_approved;
	$openid_approved = true;
}

function openid_username($openid) {
	$patterns = array();
	$replacements = array();

	$patterns[] = '/http[s]?:\/\/(.+)[\/\~\:\=](.+)\//';
	$replacements[] = '$2';

	$patterns[] = '/http[s]?:\/\/(.+)[\/\~\:\=](.+)/';
	$replacements[] = '$2';

	$patterns[] = '/http[s]?:\/\/(\w+)\.(.+)\//';
	$replacements[] = '$1';

	$patterns[] = '/\=(.+)\*(.+)/';
	$replacements[] = '$2';

	$patterns[] = '/\=(.+)/';
	$replacements[] = '$1';

	$patterns[] = '/\@(.+)\*(.+)/';
	$replacements[] = '$2';

	$patterns[] = '/\@(.+)/';
	$replacements[] = '$1';

	return preg_replace($patterns, $replacements, $openid);
}

// ----------------------------------------------------
// update_openid_user()
//
// Adds new user to phpBB database, basing on the information
// returned from the OpenID Server.
// ----------------------------------------------------
function update_openid_user ($openid) {
	global $db, $board_config;

	// reading user informations from sreg
	$email    = isset($HTTP_GET_VARS['openid_sreg_email']) ? $HTTP_GET_VARS['openid_sreg_email'] : '';
	$location = isset($HTTP_GET_VARS['openid_sreg_country']) ? $HTTP_GET_VARS['openid_sreg_country'] : '';

	if ($email) {
		$email = " user_email =  '" . str_replace("\'", "''", $email) . "', ";
	}
	if ($location) {
		$location = "user_from = '" . str_replace("\'", "''", $location) . "', ";
	}

	// Update user
	$sql = "UPDATE " . USERS_TABLE . ' SET '
		. $email
		. $location;
	$sql = substr($sql,0,strlen($sql) - 2);
	$sql = $sql . " WHERE username = '" . $openid->OpenID_Standarize($openid->GetIdentity()) . "'";

	if ( !($result = $db->sql_query($sql)) ) {
		message_die(GENERAL_ERROR, 'Could not update data to users table', '', __LINE__, __FILE__, $sql);
	}

}

function openid_migrate($user_id, $username, $email, $openid)
{
	global $board_config, $db, $template, $lang;
	global $phpbb_root_path, $phpEx;
	
	$script_name = preg_replace('/^\/?(.*?)\/?$/', '\1', trim($board_config['script_path']));
	$script_name = ( $script_name != '' ) ? $script_name . '/profile.'.$phpEx : 'profile.'.$phpEx;
	$server_name = trim($board_config['server_name']);
	$server_protocol = ( $board_config['cookie_secure'] ) ? 'https://' : 'http://';
	$server_port = ( $board_config['server_port'] <> 80 ) ? ':' . trim($board_config['server_port']) . '/' : '/';

	$server_url = $server_protocol . $server_name . $server_port . $script_name;

	$unhtml_specialchars_match = array('#&gt;#', '#&lt;#', '#&quot;#', '#&amp;#');
	$unhtml_specialchars_replace = array('>', '<', '"', '&');

	$user_actkey = md5(dss_rand());
	$key_len = 54 - strlen($server_url);
	$key_len = ( $key_len > 6 ) ? $key_len : 6;
	$user_actkey = substr($user_actkey, 0, $key_len);

	$sql = "UPDATE " . USERS_TABLE . "
		SET user_openid = '" . str_replace("\'", "''", $openid) . "', user_active = 0, user_actkey = '" . str_replace("\'", "''", $user_actkey) . "'
		WHERE user_id = $user_id";
	if ( !($result = $db->sql_query($sql)) ) {
		message_die(GENERAL_ERROR, 'Could not update users table', '', __LINE__, __FILE__, $sql);
	}

	include($phpbb_root_path . 'includes/emailer.'.$phpEx);
	$emailer = new emailer($board_config['smtp_delivery']);

	$emailer->from($board_config['board_email']);
	$emailer->replyto($board_config['board_email']);

	$emailer->use_template('user_activate', stripslashes($user_lang));
	$emailer->email_address($email);
	$emailer->set_subject($lang['Reactivate']);

	$emailer->assign_vars(array(
		'SITENAME' => $board_config['sitename'],
		'USERNAME' => preg_replace($unhtml_specialchars_match, $unhtml_specialchars_replace, substr(str_replace("\'", "'", $username), 0, 25)),
		'EMAIL_SIG' => (!empty($board_config['board_email_sig'])) ? str_replace('<br />', "\n", "-- \n" . $board_config['board_email_sig']) : '',

		'U_ACTIVATE' => $server_url . '?mode=activate&' . POST_USERS_URL . '=' . $user_id . '&act_key=' . $user_actkey)
	);
	$emailer->send();
	$emailer->reset();

	$message = $lang['Profile_updated_inactive'] . '<br /><br />' . sprintf($lang['Click_return_index'],  '<a href="' . append_sid("index.$phpEx") . '">', '</a>');
	$template->assign_vars(array(
		"META" => '<meta http-equiv="refresh" content="5;url=' . append_sid("index.$phpEx") . '">')
	);

	message_die(GENERAL_MESSAGE, $message);
}

function openid_failure($failure_message)
{
	global $template, $lang, $phpEx;
	
	$redirect = ( !empty($HTTP_POST_VARS['redirect']) ) ? str_replace('&amp;', '&', htmlspecialchars($HTTP_POST_VARS['redirect'])) : "";
	$redirect = str_replace("?", "&", $redirect);

	if (strstr(urldecode($redirect), "\n") || strstr(urldecode($redirect), "\r") || strstr(urldecode($redirect), ';url'))
	{
		message_die(GENERAL_ERROR, 'Tried to redirect to potentially insecure url.');
	}

	$template->assign_vars(array(
		'META' => "<meta http-equiv=\"refresh\" content=\"3;url=login.$phpEx?redirect=$redirect\">")
	);

	$message = $failure_message . '<br /><br />' . sprintf($lang['Click_return_login'], "<a href=\"login.$phpEx?redirect=$redirect\">", '</a>') . '<br /><br />' .  sprintf($lang['Click_return_index'], '<a href="' . append_sid("index.$phpEx") . '">', '</a>');
	message_die(GENERAL_MESSAGE, $message);
}

?>
